@@ -7,12 +7,13 @@ import random
 import re
 import requests
 
+from dbstream.tools.regex import replace_query_details
 from dbstream.tunnel import create_ssh_tunnel
 
 
 class DBStream:
 
-    def __init__(self, instance_name, client_id):
+    def __init__(self, instance_name, client_id, special_env=None):
         self.instance_name = instance_name
         self.instance_type_prefix = ""
         self.ssh_init_port = ""
@@ -20,6 +21,10 @@ class DBStream:
         self.ssh_tunnel = None
         self.dbstream_instance_id = 'df-' + datetime.datetime.now().strftime('%s') + '-' + str(
             random.randint(1000, 9999))
+        self.special_env = special_env if special_env else None
+
+    def error_if_function_not_exist(self, function_name):
+        raise Exception("Function %s is not defined for %s" % (function_name, type(self).__name__))
 
     def prefix(self):
         return self.instance_type_prefix + "_" + self.instance_name
@@ -55,11 +60,13 @@ class DBStream:
         return self.ssh_tunnel
 
     def _execute_query_custom(self, query) -> dict:
-        pass
+        self.error_if_function_not_exist("_execute_query_custom")
 
-    def execute_query(self, query):
+    def execute_query(self, query, apply_special_env=True):
         query = re.sub(' +', ' ', query)
         query = re.sub(' +\n', '\n', query)
+        if apply_special_env and self.special_env:
+            query = replace_query_details(query, self.special_env)
         result = self._execute_query_custom(query)
         if isinstance(result, dict):
             if result.get('execute_query'):
@@ -125,11 +132,23 @@ class DBStream:
             replace=False)
 
     def clean(self, selecting_id, schema_prefix, table):
-        pass
+        self.error_if_function_not_exist("clean")
 
     def send_with_temp_table(self, data, column_names, selecting_id, schema_prefix, table):
         self.send_temp_data(data, schema_prefix, table, column_names)
         self.clean(selecting_id, schema_prefix, table)
 
     def get_max(self, schema, table, field, filter_clause):
-        pass
+        self.error_if_function_not_exist("get_max")
+
+    def get_data_type(self, table_name, schema_name):
+        self.error_if_function_not_exist("get_data_type")
+
+    def create_view_from_columns(self, view_name, columns, schema_name, table_name):
+        self.error_if_function_not_exist("create_view_from_columns")
+
+    def create_schema(self, schema_name):
+        self.error_if_function_not_exist("create_schema")
+
+    def drop_schema(self, schema_name):
+        self.error_if_function_not_exist("drop_schema")
